@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-import functools
+import functools as ft
 import operator as op
 
 import jax
@@ -8,15 +8,54 @@ import jax.numpy as jnp
 import jax.tree_util as jtu
 
 from pytreeclass.src.decorator_util import dispatch
-from pytreeclass.src.tree_util import _dispatched_tree_map, is_excluded, is_treeclass
+from pytreeclass.src.tree_util import is_excluded, is_treeclass
+
+
+def _dispatched_op_tree_map(func, lhs, rhs=None, is_leaf=None):
+    """Slightly different implementation to jtu.tree_map for unary/binary operators broadcasting"""
+
+    @dispatch(argnum=1)
+    def _tree_map(lhs, rhs):
+        raise NotImplementedError(f"rhs of type {type(rhs)} is not implemented.")
+
+    @_tree_map.register(type(lhs))
+    def _(lhs, rhs):
+        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs, is_leaf=is_leaf)
+        rhs_leaves, rhs_treedef = jtu.tree_flatten(rhs, is_leaf=is_leaf)
+
+        lhs_leaves = [
+            func(lhs_leaf, rhs_leaf) if rhs_leaf is not None else lhs_leaf
+            for (lhs_leaf, rhs_leaf) in zip(lhs_leaves, rhs_leaves)
+        ]
+
+        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
+
+    @_tree_map.register(jax.interpreters.partial_eval.DynamicJaxprTracer)
+    @_tree_map.register(int)
+    @_tree_map.register(float)
+    @_tree_map.register(complex)
+    @_tree_map.register(bool)
+    @_tree_map.register(str)
+    def _(lhs, rhs):
+        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs, is_leaf=is_leaf)
+        lhs_leaves = [func(leaf, rhs) for leaf in lhs_leaves]
+        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
+
+    @_tree_map.register(type(None))
+    def _(lhs, rhs=None):
+        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs, is_leaf=is_leaf)
+        lhs_leaves = [func(lhs_node) for lhs_node in lhs_leaves]
+        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
+
+    return _tree_map(lhs, rhs)
 
 
 def _append_math_op(func):
     """binary and unary magic operations"""
 
-    @functools.wraps(func)
+    @ft.wraps(func)
     def wrapper(self, rhs=None):
-        return _dispatched_tree_map(func, self, rhs)
+        return _dispatched_op_tree_map(func, self, rhs)
 
     return wrapper
 
@@ -43,7 +82,7 @@ def _append_math_eq_ne(func):
         else:
             return False
 
-    @functools.wraps(func)
+    @ft.wraps(func)
     def wrapper(self, rhs):
         @dispatch(argnum=1)
         def inner_wrapper(tree, where, **kwargs):
@@ -56,7 +95,7 @@ def _append_math_eq_ne(func):
         @inner_wrapper.register(type(self))
         @inner_wrapper.register(jax.interpreters.partial_eval.DynamicJaxprTracer)
         def _(self, rhs):
-            return _dispatched_tree_map(func, self, rhs)
+            return _dispatched_op_tree_map(func, self, rhs)
 
         @inner_wrapper.register(str)
         def _(tree, where, **kwargs):
@@ -165,6 +204,13 @@ def _append_math_eq_ne(func):
     return wrapper
 
 
+def _append_exception(func):
+    def exception(*args, **kwargs):
+        raise Exception(f"{func} is not supported.")
+
+    return exception
+
+
 class treeOpBase:
 
     __abs__ = _append_math_op(op.abs)
@@ -188,6 +234,7 @@ class treeOpBase:
     __ne__ = _append_math_eq_ne(op.ne)
     __neg__ = _append_math_op(op.neg)
     __not__ = _append_math_op(op.not_)
+    __or__ = _append_math_op(op.or_)
     __pos__ = _append_math_op(op.pos)
     __pow__ = _append_math_op(op.pow)
     __rshift__ = _append_math_op(op.rshift)
@@ -195,21 +242,3 @@ class treeOpBase:
     __rsub__ = _append_math_op(op.sub)
     __truediv__ = _append_math_op(op.truediv)
     __xor__ = _append_math_op(op.xor)
-
-    def __or__(lhs, rhs):
-        @dispatch(argnum=0)
-        def or_func(x, y):
-            return x or y
-
-        @or_func.register(jnp.ndarray)
-        def lhs_array(x, y):
-            if jnp.array_equal(x, jnp.array([])):
-                return y if y is not None else x
-
-            elif isinstance(y, jnp.ndarray):
-                if jnp.array_equal(y, jnp.array([])):
-                    return x if x is not None else y
-                else:
-                    return jnp.logical_or(x, y)
-
-        return jtu.tree_map(or_func, lhs, rhs, is_leaf=lambda x: x is None)
