@@ -10,9 +10,10 @@ import jax.numpy as jnp
 import jax.tree_util as jtu
 import numpy as np
 
-from pytreeclass.src.decorator_util import dispatch
 from pytreeclass.src.tree_viz_util import _format_node_repr, _format_node_str
 
+PyTree = Any
+
 
 class static_value:
     def __init__(self, value):
@@ -32,7 +33,7 @@ def static_field(**kwargs):
 
 def is_treeclass(tree):
     """check if a class is treeclass"""
-    return hasattr(tree, "__tree_fields__")
+    return hasattr(tree, "__treeclass_structure__")
 
 
 def is_treeclass_leaf_bool(node):
@@ -46,15 +47,23 @@ def is_treeclass_leaf_bool(node):
 def is_treeclass_leaf(tree):
     """assert if a node is treeclass leaf"""
     if is_treeclass(tree):
-        fields = tree.__dataclass_fields__.values()
+
+        all_fields = {
+            **tree.__dataclass_fields__,
+            **tree.__dict__.get("__treeclass_fields__", {}),
+        }
 
         return is_treeclass(tree) and not any(
-            [is_treeclass(tree.__dict__[fi.name]) for fi in fields]
+            [is_treeclass(tree.__dict__[fi.name]) for fi in all_fields.values()]
         )
     else:
         return False
 
 
+def is_treeclass_non_leaf(tree):
+    return is_treeclass(tree) and not is_treeclass_leaf(tree)
+
+
 def is_treeclass_equal(lhs, rhs):
     """Assert if two treeclasses are equal"""
     lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs)
@@ -87,8 +96,8 @@ def sequential_tree_shape_eval(tree, array):
 
     # all dynamic/static leaves
     all_leaves = (
-        *tree.__tree_fields__[0].values(),
-        *tree.__tree_fields__[1].values(),
+        *tree.__treeclass_structure__[0].values(),
+        *tree.__treeclass_structure__[1].values(),
     )
     leaves = [leaf for leaf in all_leaves if is_treeclass(leaf)]
 
@@ -98,6 +107,10 @@ def sequential_tree_shape_eval(tree, array):
     return shape
 
 
+def tree_copy(tree):
+    return jtu.tree_unflatten(*jtu.tree_flatten(tree)[::-1])
+
+
 def _node_count_and_size(node: Any) -> tuple[complex, complex]:
     """Calculate number and size of `trainable` and `non-trainable` parameters
 
@@ -139,47 +152,6 @@ def _node_count_and_size(node: Any) -> tuple[complex, complex]:
     return (count, size)
 
 
-def _dispatched_tree_map(func, lhs, rhs=None):
-    """Slightly different implementation to jtu.tree_map for unary/binary operators broadcasting"""
-
-    @dispatch(argnum=1)
-    def _tree_map(lhs, rhs):
-        raise NotImplementedError(f"rhs of type {type(rhs)} is not implemented.")
-
-    @_tree_map.register(type(lhs))
-    def _(lhs, rhs):
-        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs)
-        rhs_leaves, rhs_treedef = jtu.tree_flatten(rhs)
-
-        lhs_leaves = [
-            func(lhs_leaf, rhs_leaf) if rhs_leaf is not None else lhs_leaf
-            for (lhs_leaf, rhs_leaf) in zip(lhs_leaves, rhs_leaves)
-        ]
-
-        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
-
-    @_tree_map.register(jax.interpreters.partial_eval.DynamicJaxprTracer)
-    @_tree_map.register(int)
-    @_tree_map.register(float)
-    @_tree_map.register(complex)
-    @_tree_map.register(bool)
-    @_tree_map.register(str)
-    def _(lhs, rhs):
-        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs)
-
-        lhs_leaves = [func(leaf, rhs) for leaf in lhs_leaves]
-
-        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
-
-    @_tree_map.register(type(None))
-    def _(lhs, rhs=None):
-        lhs_leaves, lhs_treedef = jtu.tree_flatten(lhs)
-        lhs_leaves = [func(lhs_node) for lhs_node in lhs_leaves]
-        return jtu.tree_unflatten(lhs_treedef, lhs_leaves)
-
-    return _tree_map(lhs, rhs)
-
-
 def _reduce_count_and_size(leaf):
     """reduce params count and params size of a tree of leaves"""
 
@@ -195,7 +167,13 @@ def _freeze_nodes(tree):
     """inplace freezing"""
     if is_treeclass(tree):
         object.__setattr__(tree, "__frozen_fields__", None)
-        for kw, leaf in tree.__dataclass_fields__.items():
+
+        all_fields = {
+            **tree.__dataclass_fields__,
+            **tree.__dict__.get("__treeclass_fields__", {}),
+        }
+
+        for kw, leaf in all_fields.items():
             _freeze_nodes(tree.__dict__[kw])
     return tree
 
@@ -205,6 +183,26 @@ def _unfreeze_nodes(tree):
     if is_treeclass(tree):
         if hasattr(tree, "__frozen_fields__"):
             object.__delattr__(tree, "__frozen_fields__")
-        for kw, leaf in tree.__dataclass_fields__.items():
+
+        all_fields = {
+            **tree.__dataclass_fields__,
+            **tree.__dict__.get("__treeclass_fields__", {}),
+        }
+
+        for kw, leaf in all_fields.items():
             _unfreeze_nodes(tree.__dict__[kw])
     return tree
+
+
+class mutableContext:
+    """Allow mutable behvior within this context"""
+
+    def __init__(self, instance):
+        assert is_treeclass(instance), "instance must be a treeclass"
+        self.instance = instance
+
+    def __enter__(self):
+        object.__setattr__(self.instance, "__immutable_treeclass__", False)
+
+    def __exit__(self, type_, value, traceback):
+        object.__setattr__(self.instance, "__immutable_treeclass__", True)
