@@ -2,7 +2,8 @@ import jax.numpy as jnp
 import pytest
 
 from pytreeclass import treeclass
-from pytreeclass.src.tree_util import is_treeclass_equal, static_value
+from pytreeclass.src.tree_op_base import _dispatched_op_tree_map
+from pytreeclass.src.tree_util import static_value
 
 
 @treeclass
@@ -69,18 +70,11 @@ def test_asdict():
     assert A.asdict() == {"a": 10, "b": 20, "c": 30, "name": "A"}
 
 
-def test_or():
-    @treeclass
-    class test:
-        a: jnp.ndarray
-        b: jnp.ndarray
+def test_dispatched_tree_map():
 
-    x = test(jnp.array([]), jnp.array([4, 5, 6]))
-    y = test(jnp.array([1, 2, 3]), jnp.array([]))
+    with pytest.raises(NotImplementedError):
 
-    assert is_treeclass_equal((x | y), test(jnp.array([1, 2, 3]), jnp.array([4, 5, 6])))
+        class A:
+            ...
 
-    # with pytest.raises(ValueError):
-    x = test(jnp.array([]), jnp.array([]))
-    y = test(jnp.array([1, 2, 3]), None)
-    assert is_treeclass_equal(x | y, test(jnp.array([1, 2, 3]), jnp.array([])))
+        _dispatched_op_tree_map(lambda x, y: x, 1, A())
